@@ -1,34 +1,54 @@
-#Step5:Calculate an IVIVC Model: Model Dependent or Independent Method  --> Model Dependent Method -->Wagner-Nelson method
-library(reshape)
-library(sciplot)
+### Step5:Calculate an IVIVC Model: Model Dependent or Independent Method  --> Model Dependent Method -->Wagner-Nelson method
+### library(reshape)
+### library(sciplot)
 
 WagNel<-function(InVVTestindex,
                  InVVRefindex, 
                  keindex,
-                 separateWindows=TRUE)
+                 separateWindows=TRUE){
+options(width=100)
 
 #calculate AUC(0~t) and AUC(0~INF)
-
-{
-
+###Step1
+zz <- file("ivivc_outputs.txt", open="wt")
+sink(zz, split=TRUE)
+description_version()
    #split dataframe into sub-dataframe
    W.data<-data.frame(pH=InVVTestindex$pH,formula.=InVVTestindex$formula.,subj=InVVTestindex$subj, 
                       time=InVVTestindex$time,conc.obs=InVVTestindex$conc.obs, FRD=InVVTestindex$FRD)
    W.data
    W.split<-split(W.data, list(W.data$pH ,W.data$formula., W.data$subj) )
 
-cat("Enter Dose value of extended release forms\n")
+cat("Enter the dose of ER formulations:\n")
 Dose <- scan(nlines=1,quiet=TRUE)
-cat("\nEnter PK parameter data file name\n")
-kename<-readline()
-kename<-paste(kename,".RData",sep="")
-load(kename)
-    
+cat("\n ER Dose =",Dose,"\n\n") 
+cat(" Load PK parameter data file automatically as follows.\n\n")
+keindex<-readRDS("ivivc_pk_values.RData")                ### use 'keindex', not 'kename'; here we will load PK parameters by default. -YJ 
+### edit(keindex)
+show(keindex);cat("\n\n")
+###
+### plot "In vitro Dissolution : Fraction of Released(%) vs. time" 
+    plotting.vitro(InVVTestindex)       ### see if we can log these plots...  -YJ
+###
+##plot "In vivo plasma concentration (Observed): Plasma conc.vs. time"
+    plotting.cp(InVVTestindex)
+
    AB<-NULL
    RD<-NULL
    pH<-NULL
    time<-NULL
-   formu<-NULL         
+   formu<-NULL
+###
+### stop computing if subj is not complete.
+###
+### cat("\n length of W.data =\n");show(length(unique(W.data$subj)));cat("\n\n")
+### cat("\n length of keindex =\n");show(length(unique(keindex$subj)));cat("\n\n")
+if(length(unique(W.data$subj))!=length(unique( keindex$subj))){
+  cat("\n\n Subjects with PK parameters are not consistent\n with subjects in-vivo data.\n\n")
+  readline(" ivivc for R will stop computing right now. Press Enter to stop.")
+  stop(call. = FALSE)
+}
+###
       #calculate AUC, F(t) and Fab
       for (j in 1:length(W.split)){
            #if subject of W.split==subject of kepar, then use ke of kepar to claculate AUC(0~INF)
@@ -66,22 +86,24 @@ cat("*               (time = 0 to infinity)
 cat("* Fobs(t): observed absorption rate                                        *\n")
 cat("* FABobs: observed cumulative absorption fraction(%)                       *\n")
 cat("* FRD: cumulative released fraction(%)                                     *\n")
-cat("****************************************************************************\n")
-cat("\n\n")                  
-                  #Output
-                  cat("<< Output >>\n")
-                  output<-data.frame(W.split[[j]][["pH"]],W.split[[j]][["subj"]],W.split[[j]][["formula."]],W.split[[j]][["time"]],W.split[[j]][["conc.obs"]],auc, Ft, Fab,W.split[[j]][["FRD"]])
-                  colnames(output)<-list("pH","subj","formula.","time","conc.obs","AUCobs(0~t)", "Fobs(t)", "FABobs","FRD")
-                  show(output)
-                  cat("\n<<AUCobs(0~inf) is computed with trapezoidal method>>\n\n")
-                  show (aucINF)
-                  cat("\n\n")
-                  AB[[j]]<-c(Fab)
-                  RD[[j]]<-c(W.split[[j]][["FRD"]])                                        
-                  time[[j]]<-c(W.split[[j]][["time"]])
-                  pH[[j]]<-c(W.split[[j]][["pH"]])
-                  formu[[j]]<-c(as.character(W.split[[j]][["formula."]])) 
-                   }               
+cat("****************************************************************************\n\n")
+
+#Output
+cat("<< Output >>\n")
+output<-data.frame(W.split[[j]][["pH"]],W.split[[j]][["subj"]],W.split[[j]][["formula."]],W.split[[j]][["time"]],W.split[[j]][["conc.obs"]],auc, Ft, Fab,W.split[[j]][["FRD"]])
+colnames(output)<-list("pH","subj","formula.","time","conc.obs","AUCobs(0~t)", "Fobs(t)", "FABobs","FRD")
+show(output)
+### write.csv(output,file="calc_ivivc_outputs.csv",append=TRUE,row.names=FALSE)
+cat("\n<<AUCobs(0~inf) is computed with trapezoidal method>>\n\n")
+show (aucINF)
+cat("\n\n")
+AB[[j]]<-c(Fab)
+RD[[j]]<-c(W.split[[j]][["FRD"]])                                        
+time[[j]]<-c(W.split[[j]][["time"]])
+pH[[j]]<-c(W.split[[j]][["pH"]])
+formu[[j]]<-c(as.character(W.split[[j]][["formula."]])) 
+ }
+readline(" Press Enter to continue...")               
 #use "melt" function of reshape package to melt lists (Fab and FRD, respectively) from 2*3*3 dataframe
 YY<-melt(AB)
 XX<-melt(RD)
@@ -106,29 +128,36 @@ Intercept<-coef(lm(Y~X))[1]
 Slope<-coef(lm(Y~X))[2]
 summary(wnlm<-lm( Y~X))$r.sq
 #plot in vitro-in vivo correlation plot
-iviv<-data.frame(FAB=Y,FRD=X, formula.=BB$value)
-I.split<-split(iviv, list(iviv$formula.)) 
 
+iviv<-data.frame(FAB=Y,FRD=X, formula.=BB$value)
+I.split<-split(iviv, list(iviv$formula.))
+
+par(mfrow=c(1,1), ask=TRUE)
 windows(record = TRUE )
-  
-  #F۰ʲC
+###  
+### to generate color auomatically
+###
    x<-NULL
    for(i in 1:length(I.split)){
     x[i]<-i
          }
 z <- lm(FAB~FRD, data=iviv)
-plot(vivo$FRD, vivo$FAB, group=vivo$formula., xlab="Fraction of Released (%)",ylab="Fraction of Absorption (%)",
-     col=x, bty="l", las=1, font.lab=2,cex.axis=1,cex.main=1)
-mtext("In-vitro-in-vivo-correlation Model",side=3,cex=2)  #mtext:iNr[bϪ|P,cexrjp   
-#text:bϧΤWi{R-squared and formula 
-text(10,80,paste("R-squared=",formatC(summary(wnlm<-lm( Y~X))$r.sq)) ) #catch R-squared value 
+plot(vivo$FRD, vivo$FAB, group=vivo$formula., xlab="Released Fraction (%)",ylab="Absorption Fraction (%)",
+     col=x, bty="l", las=1, font.lab=2,cex.axis=1,cex.main=1,lwd=2)
+mtext("In-vitro in-vivo Correlation Model\n",side=3,cex=2)  #mtext:iNr[bϪ|P,cexrjp   
+
+text(15,80,paste("R-squared=",formatC(summary(wnlm<-lm( Y~X))$r.sq)) ) #catch R-squared value 
 text(10,75,paste("Y=",formatC(coef(lm( iviv$FAB~iviv$FRD))[1])) )
 text(25,75,paste("+",formatC(coef(lm( iviv$FAB~iviv$FRD))[2]),"X") )
 abline(z)  # equivalent to abline(reg = z) or 
-abline(coef = coef(z))
+### abline(coef = coef(z))
+LLegend<-paste("",InVVTestindex$formula.,sep="")
+LLegend<-unique(LLegend)
+temp <- legend(100,40,legend = LLegend, box.col="white",                ### add formulation legend here
+               text.width = strwidth("1000"),
+               lty = 0, xjust = 1, yjust = 1,col=x,pch=1,lwd=2)
 
-
- #split dataframe into sub-dataframe
+#split dataframe into sub-dataframe
 W.data<-data.frame(pH=InVVTestindex$pH,formula.=InVVTestindex$formula.,subj=InVVTestindex$subj, time=InVVTestindex$time, conc.obs=InVVTestindex$conc.obs, FRD=InVVTestindex$FRD)
 W.data
 W.split<-split(W.data, list(W.data$pH ,W.data$formula., W.data$subj) )
@@ -212,20 +241,22 @@ cat("* AUCpred(0~inf): area under the predicted plasma concentration time curve
 cat("*                (time = 0 to infinity)                                    *\n")
 cat("* conc.pred: predicted plasma concentration                                *\n")
 cat("* FABpred: predicted cumulative absorption fraction(%)                     *\n")
-cat("****************************************************************************\n")
-cat("\n")    
-             #Output
-                  cat("<< Predicted Output >>\n")
-                  output<-data.frame(W.split[[j]][["pH"]],W.split[[j]][["subj"]],W.split[[j]][["formula."]],W.split[[j]][["time"]], PFab, PCp, Pauc )
-                  colnames(output)<-list("pH","subj","formula.","time","FABpred", "conc.pred", "AUCpred")
-                  show(output)
-                   cat("\n<<AUCpred(0~inf) is computed with trapezoidal method>>\n\n")
-                  show(PaucINF)
-                  cat("\n\n")
-                  PredCp[[j]]<-c(PCp)                                        
-                  time[[j]]<-c(W.split[[j]][["time"]])
-                  formu[[j]]<-c(as.character(W.split[[j]][["formula."]])) 
-                }
+cat("****************************************************************************\n\n")
+
+#Output
+     cat("<< Predicted Output >>\n")
+     output<-data.frame(W.split[[j]][["pH"]],W.split[[j]][["subj"]],W.split[[j]][["formula."]],W.split[[j]][["time"]], PFab, PCp, Pauc )
+     colnames(output)<-list("pH","subj","formula.","time","FABpred", "conc.pred", "AUCpred")
+     show(output)
+     ### write.csv(output,file="predicted_ivivc_outputs.csv",append=TRUE,row.names=FALSE)
+     cat("\n<<AUCpred(0~inf) is computed with trapezoidal method>>\n\n")
+     show(PaucINF)
+     cat("\n\n")
+     PredCp[[j]]<-c(PCp)                                        
+     time[[j]]<-c(W.split[[j]][["time"]])
+     formu[[j]]<-c(as.character(W.split[[j]][["formula."]])) 
+   }
+readline(" Press Enter to continue...")
 #for plot predicted Cp
 CC<-melt(PredCp)
 DD<-melt(time)
@@ -234,77 +265,91 @@ Predvivo<-data.frame(conc.pred=CC$value, formula.=EE$value, time=DD$value)
 cat("****************************************************************************\n")
 cat("* Step5: Evaluate an IVIVC model: Prediction Error                         *\n")
 cat("*--------------------------------------------------------------------------*\n")
-cat("* PECmax: average absolute prediction error of Cmax (%)                    *\n")
-cat("* PEAUC: average absolute prediction error of AUC (%)                      *\n")
+cat("* PE_Cmax: average absolute prediction error of Cmax (%)                   *\n")
+cat("* PE_AUC: average absolute prediction error of AUC (%)                     *\n")
 cat("****************************************************************************\n")
 cat("\n")
 cat("<<Summary: Validation report>>\n")
 Y<-data.frame(pH=pH, formula.=formula., PECmax=PEC*100, PEAUC=PEA*100)
 Y
 XX<-(aggregate(Y, by=list(pH=Y$pH,formula.=Y$formula.), mean)  ) 
-ZZ<-data.frame(pH=XX[1], formula.=XX[2],PECmax=XX[5], PEAUC=XX[6])
-show(ZZ)  
+ZZ<-data.frame(pH=XX[1], formulation=XX[2],PE_Cmax=XX[5], PE_AUC=XX[6])
+colnames(ZZ)<-list("pH", "Formulation", "  PE_Cmax", "PE_AUC")
+show(ZZ)
+write.csv(ZZ, file="validation_summary.csv",row.names=FALSE)
 cat("\n")
 cat("****************************************************************************\n")
 cat("*<<Plots >>                                                                *\n")
-cat("* Fitting Plots                                                            *\n")
-cat("* In-vitro-in-vivo-correlation Model (linear regression)                   *\n")
-cat("* Fraction of in vitro Released(%) vs. time                                *\n")
-cat("* Observed plasma concentration vs. time                                   *\n")
-cat("* Fraction of Absorption(%) vs. time                                       *\n")
-cat("* Predicted plasma concentration vs. time                                  *\n")
+cat("* - Fitting Plots                                                          *\n")
+cat("* - In-vitro-in-vivo-correlation Model (linear regression)                 *\n")
+cat("* - Fraction of in vitro Released(%) vs. time                              *\n")
+cat("* - Observed plasma concentration vs. time                                 *\n")
+cat("* - Fraction of Absorption(%) vs. time                                     *\n")
+cat("* - Predicted plasma concentration vs. time                                *\n")
 cat("****************************************************************************\n")
 cat("\n")
 
-##plot "In vivo plasma concentration (Observed): Plasma conc.vs. time"
-    plotting.cp(InVVTestindex)
 ##plot "In vivo Absorption : Fraction of Absorption(%) vs. time"  
       y<-aggregate(vivo, by=list(pH=vivo$pH,formula.=vivo$formula.,time=vivo$time ), mean)
+      yy<-data.frame(pH=vivo$pH,formula.=vivo$formula.,time=vivo$time,FAB=vivo$FAB)
       s<-data.frame(pH=y[1],formula.=y[2],time=y[3],meanFAB=y[7])
       S.data<-data.frame(pH=s$pH,formula.=s$formula.,time=s$time, meanFAB=s$FAB)
       S.split<-split(S.data, list(S.data$pH,S.data$formula.)) 
       F.split<-split(S.data, list(S.data$formula.)) 
       A.split<-split(S.split, list(S.data$pH) ) 
-       #F.split[1]-->X formulation=L
-       #F.split[[1]][["formula."]][1] -->X formulation=LĤ@ 
-       #A.split[1]-->XpH=1.2 
-       #A.split[[1]][[1]]--> XpH=1.2 & formulation=L 
-       #A.split[[1]][[1]]$pH-->  XpH=1.2 & formulation=L pH 
+
 ##plot "In vivo Absorption : Fraction of Absorption(%) vs. time"
  Predvivo<-data.frame(conc.pred=CC$value, formula.=EE$value, time=DD$value)
  P.split<-split(Predvivo, list(Predvivo$formula.)) 
 
-  #F۰ʲC
-   x<-NULL
-   for(i in 1:length(P.split)){
-    x[i]<-i
+### color the plots
+x<-NULL
+for(i in 1:length(P.split)){
+   x[i]<-i
          }
-            lineplot.CI(Predvivo$time, Predvivo$conc.pred, group = Predvivo$formula., cex = 1,
-            xlab = "Time", ylab = "Plasma conc.",cex.lab = 1, x.leg = 12, col=x,bty="l", 
-            font.lab=2,cex.axis=1,cex.main=1,las=1,x.cont=TRUE,xaxt="n",err.width=0.05
-             )
-            axis(1,at=c(0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,100),las=0)
-            axis(1,at=0:100,tcl=-.2, labels=FALSE) 
-            mtext("Predicted drug plasma concentration ",side=3,cex=2)  #nbplot
+    lineplot.CI(Predvivo$time, Predvivo$conc.pred, group = Predvivo$formula., cex = 1, lty=1,  ### set 'lty=1' for better looking. -YJ
+    xlab = "Time", ylab = "Drug plasma conc.",cex.lab = 1, x.leg = 12, col=x,bty="l", 
+    font.lab=2,cex.axis=1,cex.main=1,las=1,x.cont=TRUE,xaxt="n",err.width=0.05,lwd=2
+     )
+    axis(1,at=c(0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,100),las=0)
+    axis(1,at=0:100,tcl=-.2, labels=FALSE) 
+    mtext("Predicted Drug Plasma Conc.\n",side=3,cex=2)  ### must be placed after plot()
 
-##plot "In vitro Dissolution : Fraction of Released(%) vs. time" 
-    plotting.vitro(InVVTestindex)
+ for(i in 1:length(A.split)){
+    lineplot.CI(yy$time, yy$FAB, group = yy$formula., cex = 1, lty=1,    ### set 'lty=1' for better looking. -YJ
+       xlab = "Time", ylab = "Mean Predicted Fraction of Absorption (%)",cex.lab = 1,x.leg = 15,y.leg=20,col=c(1:10),bty="l",  ### max. 10 different formulations are allowed.
+       font.lab=2,cex.axis=1,cex.main=1,las=1,x.cont=TRUE,xaxt="n",err.width=0.05,lwd=2)
+       axis(1,at=c(0,2,4,6,8,10,12,14,16,18,20,24,26,28,30,32,34,36,38,40,48,72,96,120),las=0)
+       axis(1,at=0:120,tcl=-.2, labels=FALSE)
+       mtext(paste(c("In-Vivo Absorption, pH=", A.split[[i]][[i]]$pH[1],"\n"),collapse=" "),side=3,cex=2)  #must be placed after plot()
     
-par(mfrow=c(2,2))
- for(i in 1:length(A.split)){ 
-   
-      for( j in seq_along(F.split)){ 
-         main<-paste(c("In vivo Absorption pH=", A.split[[i]][[i]]$pH[1],"formula.=",as.character(F.split[[j]][["formula."]][1]) ),collapse=" ")
-         # plot points
-         plot(A.split[[i]][[j]]$time,A.split[[i]][[j]]$meanFAB,type="punkte",main=main,
-         xlab="Time", ylab="Fraction of Absorption (%)",pch=15,col=j,bty="l",las=1,
-         font.lab=2,cex.lab=1,cex.axis=1,cex.main=1)
-         #plot line
-         lines(A.split[[i]][[j]]$time, A.split[[i]][[j]]$meanFAB, col=j) 
-   }
- }  
+      ###    main<-paste(c("In Vivo Absorption, pH=", A.split[[i]][[i]]$pH[1]),collapse=" ")
+      ###    plot(0,xlim=range(y$time), ylim=c(0,max(y[7])),legend=FALSE,main=main,
+      ###    xlab="Time", ylab="Mean Predicted Fraction of Absorption (%)",pch=15,col="white",bty="l",las=1,
+      ###    font.lab=2,cex.lab=1,cex.axis=1,cex.main=2,lwd=2)
+      ### for(j in seq_along(F.split)){ 
+      ###    # plot points
+      ###    points(A.split[[i]][[j]]$time,A.split[[i]][[j]]$meanFAB,type="punkte",main=main,pch=(14+j),col=j,lwd=2)
+      ###    #plot line
+      ###    lines(A.split[[i]][[j]]$time, A.split[[i]][[j]]$meanFAB,lty=1,col=j,lwd=2) 
+      ###    ###
+      ###  } 
+      ###   temp <- legend(15,20, legend = c(unique(as.character(Predvivo$formula.))),
+      ###                  col=c(1:10),pch=c(15:24),text.width = strwidth("1000"),lty=1,lwd=2,    ### inappropriate legend position may cause legend not showing. -YJ
+      ###                  xjust=0,yjust=0,box.col="white")                                       ### max. 10 different formulations are allowed.
+    }  
 cat("\n")
-bye()
+filepath<-getwd()            
+sink()
+close(zz)
+ivivc_all_plots(InVVTestindex, InVVRefindex, keindex, Dose, separateWindows=TRUE)
+readline(" Press Enter to finish this run...")
+graphics.off()
+       cat("*****************************************************************************\n\n")
+       cat("## Please note: output files: ivivc_outputs.txt and ivivc_plots.pdf            \n")
+       cat("   have been created and placed at ",filepath,                               "\n\n")
+       cat("*****************************************************************************\n\n")    
+run()
 }   
     
 
